
import random, os, sys

from searcher import S_DIR, adv_parse
from results0 import fix


def run():
    # Gather raw data from the experiment
    task_data = dict()
    for file_name in sorted(os.listdir(S_DIR)):
        file_name = os.path.abspath(os.path.join(S_DIR, file_name))
        assert os.path.isfile(file_name)

        mode = file_name[-3:]
        if mode in ("che", "sel", "cwb"):
            adv_parse(task_data, file_name, mode)

    # Tasks considered important if there is Carousel data for them
    important = []
    for ((mode, task_name), td) in task_data.iteritems():
        if (mode == "sel") and (task_name.startswith("USR ")):
            important.append(task_name)

    print "CS1", max(task_data[("sel", "SYS sch from")].wcets)
    print "CS2", max(task_data[("sel", "SYS sch to")].wcets)

    # Check Carousel property (BCET = WCET)
    err = False
    for task_name in important:
        td = task_data[("sel", task_name)]
        et = td.wcets[0]
        assert len(td.wcets) == len(td.bcets)
        assert et > 0
        for i in xrange(len(td.wcets)):
            if td.bcets[i] != et:
                print "error, BCET %u != %u, run %s task %s" % (
                        td.bcets[i], et, td.taskset[i], task_name)
                err = True
            if td.wcets[i] != et:
                print "error, WCET %u != %u, run %s task %s" % (
                        td.wcets[i], et, td.taskset[i], task_name)
                err = True

    if err:
        sys.exit(1)

    # Fixup BCETs (remove zeroes)
    for ((mode, task_name), td) in task_data.iteritems():
        completes = sum(td.completes)
        td.bcets.sort()
        td.bcets.reverse()
        while ((len(td.bcets) != 0) and td.bcets[-1] <= 0):
            td.bcets.pop()


    important.sort()

    out = file("res/chewy.tex", "wt")
    out.write(r"""
\begin{tabular}{|l|r|r|r|}
\hline
Task Name   & Observed      & Observed      & Preemption \\
            & BCET          & WCET          & count \\
\hline
""")
    for task_name in important:
        td = task_data[("sel", task_name)]
        (bc, wc) = (min(td.bcets), max(td.wcets))
        out.write(r"%s & %u & %u & %u \\" % (
            fix(task_name[4:]), bc, wc,
            sum(td.preempts)))
        out.write("\n")
        assert bc == wc

    out.write(r"""
\hline
\end{tabular}
""")
    out.close()

    print 'num tasks =', len(important)

    for (out_name, src) in [("poppy", "che"), ("moggy", "cwb")]:
        out = file("res/%s.tex" % out_name, "wt")
        out.write(r"""
\begin{tabular}{|l|r|r|r|}
\hline
Task Name   & Observed      & Observed      & Comparison \\
            & BCET          & WCET          & factor \\
\hline
""")
        carousel_wcet_score = carousel_bcet_score = 0
        for task_name in important:
            td = task_data[(src, task_name)]
            td2 = task_data[("sel", task_name)]
            wcet_comp_fact = float(max(td2.wcets)) / float(max(td.wcets))
            bcet_comp_fact = float(max(td2.bcets)) / float(max(td.bcets))
            out.write(r"%s & %u & %u & %1.2f \\" % (
                fix(task_name[4:]), min(td.bcets), max(td.wcets),
                1.0 / wcet_comp_fact))
            out.write("\n")

            if wcet_comp_fact <= 1.0:
                carousel_wcet_score += 1
            if bcet_comp_fact <= 1.0:
                carousel_bcet_score += 1

        out.write(r"""
\hline
\end{tabular}
""")
        out.close()
        print src
        print 'carousel_wcet_score =', carousel_wcet_score 
        print 'carousel_bcet_score =', carousel_bcet_score 

    out = file("res/graph.csv", "wt")
    out.write(",Carousel,Cache BCET,Cache WCET\n")
    for task_name in reversed(important):
        td = task_data[("sel", task_name)]
        norm = float(max(td.wcets))

        td = task_data[("che", task_name)]
        out.write("%s,1,%1.6f,%1.6f,%1.6f\n" % (fix(task_name),
                    float(min(td.bcets)) / norm,
                    float(max(td.wcets)) / norm,
                    norm))
    out.close()

if __name__ == "__main__":
    run()

